#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright (c) 2024, Intel Corporation.

"""
For receiving, decoding and printing debug-stream records over debug window slot.
"""

import argparse
import ctypes
import time
import sys
import os

import logging

logging.basicConfig(
    format="%(filename)s:%(lineno)s %(funcName)s: %(message)s", level=logging.WARNING
)

DEBUG_STREAM_PAYLOAD_MAGIC = 0x1ED15EED
DEBUG_SLOT_SIZE = 4096

# TODO: python construct would probably be cleaner than ctypes structs

class DebugStreamHdr(ctypes.Structure):
    """
    Generic Debug-stream header
    """

    _fields_ = [
        ("magic", ctypes.c_uint),
        ("hdr_size", ctypes.c_uint),
    ]


class DebugStreamRecord(ctypes.Structure):
    """
    Debug Stream record for passing debug data
    """

    _fields_ = [
        ("id", ctypes.c_uint),
        ("seqno", ctypes.c_uint),
        ("size_words", ctypes.c_uint),
    ]


class CPUInfo(ctypes.Structure):
    """
    Thread Info record header
    """

    _pack_ = 1
    _fields_ = [
        ("hdr", DebugStreamRecord),
        ("load", ctypes.c_ubyte),
        ("thread_count", ctypes.c_ubyte),
    ]


class ThreadInfo(ctypes.Structure):
    """
    Thread specific data-record, the thread name string starts after name_len
    """

    _pack_ = 1
    _fields_ = [
        ("stack_usage", ctypes.c_ubyte),
        ("cpu_load", ctypes.c_ubyte),
        ("name_len", ctypes.c_ubyte),
    ]


WSIZE = ctypes.sizeof(ctypes.c_uint)


class RecordPrinter:
    """
    Debug Stream record decoder and printer class
    """

    RECORD_ID_UNINITIALIZED = 0
    RECORD_ID_THREAD_INFO = 1

    def print_record(self, record, cpu):
        """prints debug-stream record"""
        recp = ctypes.cast(record, ctypes.POINTER(DebugStreamRecord))
        logging.debug(
            "rec: %u %u %u", recp.contents.id, recp.contents.seqno, recp.contents.size_words
        )
        if recp.contents.id == self.RECORD_ID_THREAD_INFO:
            return self.print_thread_info(record, cpu)
        logging.warning("cpu %u: Unsupported recodrd type %u", cpu, recp.contents.id)
        return True

    def print_thread_info(self, record, cpu):
        """prints thread-info record"""
        remlen = len(record) - ctypes.sizeof(CPUInfo)
        if remlen < 0:
            logging.info("Buffer end reached, parsing failed")
            return False
        cpup = ctypes.cast(record, ctypes.POINTER(CPUInfo))
        print(
            "CPU %u: Load: %02.1f%% %u threads (seqno %u)"
            % (
                cpu,
                cpup.contents.load / 2.55,
                cpup.contents.thread_count,
                cpup.contents.hdr.seqno,
            )
        )
        remain = (ctypes.c_ubyte * (len(record) - ctypes.sizeof(CPUInfo))).from_address(
            ctypes.addressof(record) + ctypes.sizeof(CPUInfo)
        )
        for i in range(cpup.contents.thread_count):
            remlen = remlen - ctypes.sizeof(ThreadInfo)
            if remlen < 0:
                logging.info("Buffer end reached, parsing failed on %u thread field", i)
                return False
            threadp = ctypes.cast(remain, ctypes.POINTER(ThreadInfo))
            remain = (
                ctypes.c_ubyte * (len(remain) - ctypes.sizeof(ThreadInfo))
            ).from_address(ctypes.addressof(remain) + ctypes.sizeof(ThreadInfo))
            remlen = remlen - threadp.contents.name_len
            if remlen < 0:
                logging.info("Buffer end reached, parsing failed on %u thread field", i)
                return False
            name = bytearray(remain[: threadp.contents.name_len]).decode("utf-8")
            remain = (
                ctypes.c_ubyte * (len(remain) - threadp.contents.name_len)
            ).from_address(ctypes.addressof(remain) + threadp.contents.name_len)
            print(
                "    %-20s stack %02.1f%%\tload %02.1f%%"
                % (
                    name,
                    threadp.contents.stack_usage / 2.55,
                    threadp.contents.cpu_load / 2.55,
                )
            )
        return True


class DebugStreamSectionDescriptor(ctypes.Structure):
    """
    Describes CPU specific circular buffers
    """

    _fields_ = [
        ("core_id", ctypes.c_uint),
        ("buf_words", ctypes.c_uint),
        ("offset", ctypes.c_uint),
    ]


class DebugStreamSlotHdr(ctypes.Structure):
    """
    Debug Slot transport specific Debug Stream header, padded to cache line
    """

    _fields_ = [
        ("hdr", DebugStreamHdr),
        ("total_size", ctypes.c_uint),
        ("num_sections", ctypes.c_uint),
    ]


class CircularBufHdr(ctypes.Structure):
    """
    Live data header for CPU specific circular buffer
    """

    _fields_ = [
        ("next_seqno", ctypes.c_uint),
        ("w_ptr", ctypes.c_uint),
    ]


class CircularBufferDecoder:
    """
    Class for extracting records from circular buffer
    """

    desc = None
    boffset = None
    buf_words = None
    cpu = None
    printer = None
    prev_w_ptr = 0
    prev_seqno = None
    error_count = 0

    def __init__(self, desc, cpu, printer):
        self.desc = desc
        self.boffset = desc.offset + ctypes.sizeof(CircularBufHdr)
        self.buf_words = desc.buf_words
        self.cpu = cpu
        self.printer = printer
        logging.debug(
            "boffset %u buf_words %u cpu %u", self.boffset, self.buf_words, self.cpu
        )

    def get_hdr(self, slot, pos):
        """
        Get record header from position (handles circular buffer wrap)
        """
        if pos >= self.buf_words:
            logging.warning("Bad position %u", pos)
            return None
        hdr_size = ctypes.sizeof(DebugStreamRecord)
        hdr_words = hdr_size // WSIZE
        if pos + hdr_words > self.buf_words:
            hdr = (ctypes.c_ubyte * hdr_size)()
            size1 = (self.buf_words - pos) * WSIZE
            size2 = hdr_size - size1
            pos1 = self.boffset + pos * WSIZE
            pos2 = self.boffset
            logging.debug(
                "Wrapped header %u %u %u %u", pos, hdr_words, self.buf_words, size1
            )

            hdr[0:size1] = slot[pos1 : pos1 + size1]
            hdr[size1:hdr_size] = slot[pos2 : pos2 + size2]
            header = ctypes.cast(hdr, ctypes.POINTER(DebugStreamRecord)).contents
        else:
            header = ctypes.cast(
                slot[self.boffset + pos * WSIZE :], ctypes.POINTER(DebugStreamRecord)
            ).contents
        if header.id > 100 or header.size_words >= self.buf_words:
            logging.info(
                "Broken record id %u seqno %u size %u",
                header.id,
                header.seqno,
                header.size_words,
            )
            return None
        return header

    def get_record(self, slot, pos, seqno):
        """
        Get record from position
        """
        rec = self.get_hdr(slot, pos)
        if rec is None or rec.size_words == 0:
            return None
        logging.debug(
            "got header at pos %u rec %u %u %u", pos, rec.id, rec.seqno, rec.size_words
        )
        if seqno is not None and rec.seqno != seqno:
            logging.warning(
                "Record seqno mismatch %u != %u, pos %u size %u",
                rec.seqno,
                seqno,
                pos,
                rec.size_words,
            )
            self.error_count = self.error_count + 1
            return None
        rwords = rec.size_words
        rsize = rec.size_words * WSIZE
        if pos + rwords > self.buf_words:
            record = (ctypes.c_ubyte * rsize)()
            size1 = (self.buf_words - pos) * WSIZE
            size2 = rsize - size1
            pos1 = self.boffset + pos * WSIZE
            pos2 = self.boffset
            logging.debug(
                "Wrapped record %u %u %u %u", pos, rsize, self.buf_words, size1
            )

            record[0:size1] = slot[pos1 : pos1 + size1]
            record[size1:rsize] = slot[pos2 : pos2 + size2]
        else:
            record = (ctypes.c_ubyte * rsize).from_buffer_copy(
                slot, self.boffset + pos * WSIZE
            )
        logging.info("got %u", rec.seqno)
        self.error_count = 0
        return record

    def catch_up(self, slot):
        """
        Search backwards from w_ptr for valid records
        """
        circ = CircularBufHdr.from_buffer_copy(slot, self.desc.offset)
        if circ.next_seqno == 0 or circ.w_ptr >= self.buf_words:
            return
        self.decode_past_records(slot, circ.w_ptr, circ.next_seqno)
        self.prev_w_ptr = circ.w_ptr
        self.prev_seqno = circ.next_seqno - 1
        logging.debug("seqno %u w_ptr %u", self.prev_seqno, self.prev_w_ptr)

    def decode_past_records(self, slot, pos, seqno):
        """
        Decode records backwards from pos. Should not be called directly, use catch_up()
        """
        if self.prev_seqno is not None and self.prev_seqno >= seqno - 1:
            return
        if pos == 0:
            spos = self.buf_words - 1
        else:
            spos = pos - 1
        bsize = ctypes.cast(
            slot[self.boffset + spos * 4 :], ctypes.POINTER(ctypes.c_uint)
        ).contents.value
        bpos = pos - bsize
        if bpos < 0:
            bpos = self.buf_words + pos - bsize
        rec = self.get_hdr(slot, bpos)
        if rec == None:
                return
        if bsize != rec.size_words:
            return
        if seqno is not None:
            if rec.seqno != seqno - 1:
                return
        else:
            seqno = rec.seqno + 1

        self.decode_past_records(slot, bpos, seqno - 1)

        record = self.get_record(slot, bpos, seqno - 1)
        if record is not None:
            if not self.printer.print_record(record, self.cpu):
                logging.info("Parse failed on record %u", seqno - 1)
            logging.info("Printing %u success", seqno - 1)
        else:
            logging.info("Broken record %u", seqno - 1)

    def get_next_record(self, slot):
        """
        Get next record from the circular buffer and print it, returns True if a record
        with expected seqno number was found and successfully decoded and printed.
        """
        if self.prev_seqno is not None:
            record = self.get_record(slot, self.prev_w_ptr, self.prev_seqno + 1)
        else:
            record = self.get_record(slot, self.prev_w_ptr, None)
        if record is not None:
            print_success = self.printer.print_record(record, self.cpu)
            if print_success:
                recp = ctypes.cast(record, ctypes.POINTER(DebugStreamRecord))
                self.prev_w_ptr = (
                    self.prev_w_ptr + recp.contents.size_words
                ) % self.buf_words
                self.prev_seqno = recp.contents.seqno
            else:
                logging.info("Parse failed on record %u", self.prev_seqno + 1)
            return print_success
        self.error_count = self.error_count + 1
        logging.info("Record decoding failed %u", self.error_count)
        return False

    def poll_buffer(self, slot):
        """
        Poll for new records. If there were new records return True
        """
        circ = CircularBufHdr.from_buffer_copy(slot, self.desc.offset)
        if self.prev_w_ptr == circ.w_ptr:
            return False
        got_record = True
        while self.prev_w_ptr != circ.w_ptr and got_record:
            got_record = self.get_next_record(slot)
        return True

    def check_error_count(self):
        """
        Check if maximum error count was reached
        """
        if self.error_count > 3:
            return True
        return False


class DebugStreamDecoder:
    """
    Class for decoding debug-stream slot contents
    """

    file_size = DEBUG_SLOT_SIZE
    file = None
    slot = None
    descs = []
    circdec = []
    rec_printer = RecordPrinter()

    def set_file(self, file):
        """
        Ser file to read the slot contents from
        """
        self.file = file

    def update_slot(self):
        """
        Update slot contents
        """
        self.file.seek(0)
        self.slot = self.file.read(self.file_size)

    def set_slot(self, buf):
        """
        Update slot contents
        """
        self.slot = buf

    def get_descriptors(self):
        """
        Read the core specific descriptors and initialize core
        specific circular buffer decoders.

        """
        if self.slot is None:
            return False
        hdr = ctypes.cast(self.slot, ctypes.POINTER(DebugStreamSlotHdr))
        if hdr.contents.hdr.magic != DEBUG_STREAM_PAYLOAD_MAGIC:
            logging.info("Debug Slot has bad magic 0x%08x", hdr.contents.hdr.magic)
            return False
        num_sections = hdr.contents.num_sections
        if num_sections == len(self.descs):
            return True
        if num_sections > 32:
            logging.info("Suspiciously many sections %u", num_sections)
            return False
        hsize = ctypes.sizeof(DebugStreamSlotHdr)
        self.descs = (DebugStreamSectionDescriptor * num_sections).from_buffer_copy(
            self.slot, hsize
        )
        for i in range(len(self.descs)):
            if (self.descs[i].core_id > 32 or
                self.descs[i].buf_words > DEBUG_SLOT_SIZE // WSIZE or
                self.descs[i].offset > DEBUG_SLOT_SIZE):
             logging.info("Suspicious descriptor %u values %u %u %u", i,
                          self.descs[i].core_id, self.descs[i].buf_words,
                          self.descs[i].offset)
             return False
        self.circdec = [
            CircularBufferDecoder(self.descs[i], i, self.rec_printer)
            for i in range(len(self.descs))
        ]
        logging.info(
            "Header hdr_size %u total_size %u num_sections %u",
            hdr.contents.hdr.hdr_size,
            hdr.contents.total_size,
            hdr.contents.num_sections,
        )
        return True

    def catch_up_all(self):
        """
        Checks all circular buffers if there is records behind w_ptr. If there
        is valid seqno number available the decoding stops when there previous
        printed record is reached.
        """
        if len(self.descs) == 0 or self.slot is None:
            return
        for i in range(len(self.descs)):
            self.circdec[i].catch_up(self.slot)

    def poll(self):
        """
        Poll all circular buffers for more records. Returns True if nothing new
        was found and its time to go to sleep.
        """
        if len(self.descs) == 0 or self.slot is None:
            return False
        sleep = True
        for i in range(len(self.descs)):
            if self.circdec[i].poll_buffer(self.slot):
                sleep = False
        return sleep

    def check_slot(self):
        """
        Check if previously updated slot contents is valid
        """
        hdr = ctypes.cast(self.slot, ctypes.POINTER(DebugStreamSlotHdr))
        if hdr.contents.hdr.magic != DEBUG_STREAM_PAYLOAD_MAGIC:
            self.slot = None
            return False
        if hdr.contents.num_sections != len(self.descs):
            self.slot = None
            return False
        for i in range(len(self.descs)):
            if self.circdec[i].check_error_count():
                self.circdec[i] = CircularBufferDecoder(
                    self.descs[i], i, self.rec_printer
                )
        return True

    def reset(self):
        """
        Reset decoder
        """
        self.file = None
        self.slot = None

def cavstool_main_loop(my_args):
    import cavstool
    try:
        (hda, sd, dsp, hda_ostream_id) = cavstool.map_regs(True)
    except Exception as e:
        logging.error("Could not map device in sysfs; run as root?")
        logging.error(e)
        sys.exit(1)
    ADSP_DW_SLOT_DEBUG_STREAM = 0x53523134
    decoder = DebugStreamDecoder()
    while True:
        if not cavstool.fw_is_alive(dsp):
            cavstool.wait_fw_entered(dsp, timeout_s=None)
        if my_args.direct_access_slot < 0:
            offset = cavstool.debug_slot_offset_by_type(ADSP_DW_SLOT_DEBUG_STREAM, timeout_s=0.5)
            if offset is None:
                logging.error("Could not find debug_stream slot")
                sys.exit(1)
            logging.info("Got offset 0x%08x by type 0x%08x", offset,
                         ADSP_DW_SLOT_DEBUG_STREAM)
        else:
            offset = cavstool.debug_slot_offset(my_args.direct_access_slot)
        buf = cavstool.win_read(offset, 0, DEBUG_SLOT_SIZE)
        decoder.set_slot(buf)
        if not decoder.get_descriptors():
            time.sleep(my_args.update_interval)
            continue
        decoder.catch_up_all()
        while True:
            if decoder.poll():
                time.sleep(my_args.update_interval)
            buf = cavstool.win_read(offset, 0, DEBUG_SLOT_SIZE)
            decoder.set_slot(buf)
            if not decoder.check_slot():
                break

def main_f(my_args):
    """
    Open debug stream slot file and pass it to decoder

    This tool and the protocol it implements is for debugging and optimized
    to interfere with SOF DSP as little as possible, but not worrying too much
    about the host CPU load. That is why there where no synchronous mechanism
    done and the host simply polls for new records.
    """
    if my_args.direct_access_slot >= 0 or not os.path.isfile(my_args.debugstream_file):
	    return cavstool_main_loop(my_args)
    decoder = DebugStreamDecoder()
    prev_error = None
    while True:
        try:
            with open(my_args.debugstream_file, "rb") as file:
                decoder.set_file(file)
                decoder.update_slot()
                if not decoder.get_descriptors():
                    time.sleep(my_args.update_interval)
                    continue
                decoder.catch_up_all()
                while True:
                    if decoder.poll():
                        time.sleep(my_args.update_interval)
                    decoder.update_slot()
                    if not decoder.check_slot():
                        break

        except FileNotFoundError:
            print(f"File {my_args.debugstream_file} not found!")
            break
        except OSError as err:
            if str(err) != prev_error:
                print(f"Open {my_args.debugstream_file} failed '{err}'")
                prev_error = str(err)
        decoder.reset()
        time.sleep(args.update_interval)


def parse_params():
    """Parses parameters"""
    parser = argparse.ArgumentParser(description="SOF DebugStream thread info client. ")
    parser.add_argument(
        "-t",
        "--update-interval",
        type=float,
        help="Telemetry2 window polling interval in seconds, default 1",
        default=0.01,
    )
    parser.add_argument(
        "-f",
        "--debugstream-file",
        help="File to read the DebugStream data from, default /sys/kernel/debug/sof/debug_stream",
        default="/sys/kernel/debug/sof/debug_stream",
    )
    parser.add_argument(
        "-c",
        "--direct-access-slot",
        help="Access specified debug window slot directly, no need for debugfs file",
        type=int,
        default=-1,
    )
    parsed_args = parser.parse_args()
    return parsed_args


if __name__ == "__main__":
    args = parse_params()
    main_f(args)



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause
import re
import sys

# Very simple UUID registry validator and C header generator.  Parses
# uuid-registry.txt (passed as the first command line argument) and
# writes a C header (named in the second argument) containing
# definitions to be used at build time.  Fails via assertion if the
# any element in the registry is invalid.

header = """/*
 * GENERATED CODE.  DO NOT EDIT.
 *
 * Generated UUID records (initializers for struct sof_uuid)
 * See scripts/gen-uuid-reg.py
 */
#ifndef _UUID_REGISTRY_H
#define _UUID_REGISTRY_H
"""

all_syms = set()
all_uuids = set()
out_recs = []

def emit_uuid_rec(uu, sym):
    recs = uu.split('-')
    brec = recs[3]
    wrecs = [ "0x" + r for r in recs[0:3] ]
    byts = [ "0x" + brec[ 2*i : 2*i+2 ] for i in range(int(len(brec) / 2)) ]
    uuidinit = "{ " + ", ".join(wrecs) + ", { " + ", ".join(byts) + " } }"
    out_recs.append(f"#define _UUIDREG_{sym} {uuidinit}")

    uuidstr = uu[:23] + '-' + uu[23:]
    out_recs.append(f'#define UUIDREG_STR_{sym.upper()} "{uuidstr}"')

def main():
    with open(sys.argv[1]) as f:
        for line in f.readlines():
            line = re.sub(r'\s*#.*', '', line) # trim comments
            line = re.sub(r'^\s*', '', line)   # trim leading ws
            line = re.sub(r'\s*$', '', line)   # trim trailing ws
            if line == "":
                continue
            m = re.match(r'(.*)\s+(.*)', line)
            assert m
            (uu, sym) = (m.group(1).lower(), m.group(2))
            assert re.match(r'[0-9a-f]{8}(?:-[0-9a-f]{4}){2}-[0-9a-f]{16}', uu)
            assert re.match(r'[a-zA-Z_][a-zA-Z0-9_]*', sym)
            assert len(sym) < 32
            assert uu not in all_uuids
            assert sym not in all_syms
            all_uuids.add(uu)
            all_syms.add(sym)
            emit_uuid_rec(uu, sym)
            print(f"Added UUID {uu} with symbol {sym}")

    with open(sys.argv[2], "w") as f:
        f.write(header)
        for l in out_recs:
            f.write(l + "\n")
        f.write("#endif /* _UUID_REGISTRY_H */\n")

if __name__ == "__main__":
	main()



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause

# We need to calculate ELF section addresses of an LLEXT module and use them to
# run the linker. We could just use Python to calculate addresses and pass them
# back to cmake to have it call the linker. However, there doesn't seem to be a
# portable way to do that. Therefore we pass the linker path and all the command
# line parameters to this script and call the linker directly.

import os
import argparse
import subprocess
from elftools.elf.elffile import ELFFile
from elftools.elf.constants import SH_FLAGS
import re
import pathlib

args = None
def parse_args():
	global args

	parser = argparse.ArgumentParser(description='Helper utility to run a linker command '
                                         'with calculated ELF section addresses')

	parser.add_argument('command', type=str, help='Linker command to execute')
	parser.add_argument('params', nargs='+', help='Additional linker parameters')
	parser.add_argument("-f", "--file", required=True, type=str,
						help='Object file name')
	parser.add_argument("-c", "--copy", required=True, type=str,
						help='Objcopy command')
	parser.add_argument("-o", "--output", required=True, type=str,
						help='Output file name')
	parser.add_argument("-t", "--text-addr", required=True, type=str,
						help='.text section address')
	parser.add_argument("-s", "--size-file", required=True, type=str,
						help='File with stored accumulated size')

	args = parser.parse_args()

def align_up(addr, align):
	upper = addr + align - 1
	return upper - (upper % align)

def max_alignment(addr, align1, align2):
	if align2 > align1:
		align1 = align2

	upper = addr + align1 - 1
	return upper - (upper % align1)

def main():
	global args

	parse_args()

	# Get the size of the previous module, if this isn't the first one.
	# It is used to automatically calculate starting address of the current
	# module.
	try:
		with open(args.size_file, 'r') as f_size:
			size = int(f_size.read().strip(), base = 0)
	except OSError:
		size = 0

	text_addr = int(args.text_addr, 0) + size
	text_size = 0

	# File names differ when building shared or relocatable objects
	if args.file[:-3] == '.so':
		p = re.compile(r'(^lib|\.so$)')
		fname = args.file
	else:
		fpath = pathlib.Path(args.file)
		fname = fpath.name
		p = re.compile(r'(^lib|_llext_lib\.obj$)')
	module = p.sub('', fname)

	command = [args.command]

	executable = []
	writable = []
	readonly = []
	readonly_dram = []

	text_found = False

	elf = ELFFile(open(args.file, 'rb'))

	# Create an object file with sections grouped by their properties,
	# similar to how program segments are created: all executable sections,
	# then all read-only data sections, and eventually all writable data
	# sections like .data and .bss. Each group is aligned on a page boundary
	# (0x1000) to make dynamic memory mapping possible. The resulting object
	# file will either be a shared library or a relocatable (partially
	# linked) object, depending on the build configuration.
	for section in elf.iter_sections():
		s_flags = section.header['sh_flags']
		s_type = section.header['sh_type']
		s_name = section.name
		s_size = section.header['sh_size']
		s_alignment = section.header['sh_addralign']

		if not s_flags & SH_FLAGS.SHF_ALLOC:
			continue

		if (s_flags & (SH_FLAGS.SHF_ALLOC | SH_FLAGS.SHF_EXECINSTR) ==
                    SH_FLAGS.SHF_ALLOC | SH_FLAGS.SHF_EXECINSTR and
		    s_type == 'SHT_PROGBITS'):
			# An executable section.
			if s_name == '.text':
				text_found = True
				text_addr = max_alignment(text_addr, 0x1000, s_alignment)
				text_size = s_size
				command.append(f'-Wl,-Ttext=0x{text_addr:x}')
			else:
				executable.append(section)

			continue

		if (s_flags & (SH_FLAGS.SHF_WRITE | SH_FLAGS.SHF_ALLOC) ==
                    SH_FLAGS.SHF_WRITE | SH_FLAGS.SHF_ALLOC):
			# .data, .bss or other writable sections
			writable.append(section)
			continue

		if s_type == 'SHT_PROGBITS' and s_flags & SH_FLAGS.SHF_ALLOC:
			# .rodata or other read-only sections
			if s_name == '.coldrodata':
				readonly_dram.append(section)
			else:
				readonly.append(section)

	if not text_found:
		raise RuntimeError('No .text section found in the object file')

	# The original LLEXT support in SOF linked all LLEXT modules with pre-
	# calculated addresses. Such modules can only be used at those exact
	# addresses, so we map memory buffers for such modules to those
	# addresses and copy them there.
	# Now we also need to be able to re-link parts of modules at run-time to
	# run at arbitrary memory locations. One of the use-cases is running
	# parts of the module directly in DRAM - sacrificing performance but
	# saving scarce SRAM. We achieve this by placing non-performance
	# critical functions in a .cold ELF section, read-only data in a
	# .coldrodata ELF section, etc. When compiling and linking such
	# functions, an additional .cold.literal section is automatically
	# created. Note, that for some reason the compiler also marks .cold as
	# executable.
	# This script links those sections at address 0. We could hard-code
	# section names, but so far we choose to only link .text the "original"
	# way and all other executable sections we link at 0. For data sections
	# we accept only the .coldrodata name for now.

	dram_addr = 0
	first_dram_text = None
	first_dram_rodata = None

	for section in executable:
		s_alignment = section.header['sh_addralign']
		s_name = section.name

		if not first_dram_text:
			first_dram_text = s_name

		dram_addr = align_up(dram_addr, s_alignment)

		command.append(f'-Wl,--section-start={s_name}=0x{dram_addr:x}')

		dram_addr += section.header['sh_size']

	for section in readonly_dram:
		s_alignment = section.header['sh_addralign']
		s_name = section.name

		if not first_dram_rodata:
			first_dram_rodata = s_name

		dram_addr = align_up(dram_addr, s_alignment)

		command.append(f'-Wl,--section-start={s_name}=0x{dram_addr:x}')

		dram_addr += section.header['sh_size']

	start_addr = align_up(text_addr + text_size, 0x1000)

	for section in readonly:
		s_alignment = section.header['sh_addralign']
		s_name = section.name

		start_addr = align_up(start_addr, s_alignment)

		command.append(f'-Wl,--section-start={s_name}=0x{start_addr:x}')

		start_addr += section.header['sh_size']

	start_addr = align_up(start_addr, 0x1000)

	for section in writable:
		s_alignment = section.header['sh_addralign']
		s_name = section.name

		start_addr = align_up(start_addr, s_alignment)

		if s_name == '.data':
			command.append(f'-Wl,-Tdata=0x{start_addr:x}')
		else:
			command.append(f'-Wl,--section-start={s_name}=0x{start_addr:x}')

		start_addr += section.header['sh_size']

	command.extend(['-o', f'{args.file}.tmp'])
	command.extend(args.params)

	subprocess.run(command)

	copy_command = [args.copy]

	if first_dram_text:
		copy_command.extend(['--set-section-alignment', f'{first_dram_text}=4096'])
	if first_dram_rodata:
		copy_command.extend(['--set-section-alignment', f'{first_dram_rodata}=4096'])

	copy_command.extend([f'{args.file}.tmp', f'{args.output}'])
	subprocess.run(copy_command)

if __name__ == "__main__":
	main()



# SPDX-License-Identifier: BSD-3-Clause

import subprocess
import os
import signal
import re
import math

# Global variables
aplay_process = None
arecord_process = None
paused = None
current_file = None
device_string = None
volume_control = None
eq_numid = None
is_muted = False
previous_volume = 50  # Store the previous volume level
extra_audio_paths = []

def initialize_device():
    global device_string, volume_control, eq_numid

    try:
        output = subprocess.check_output(["aplay", "-l"], text=True, stderr=subprocess.DEVNULL)
        match = re.search(r"card (\d+):.*\[.*sof.*\]", output, re.IGNORECASE)
        if match:
            card_number = match.group(1)
            device_string = f"hw:{card_number}"
            print(f"Detected SOF card: {device_string}")
        else:
            raise RuntimeError("SOF card not found. Ensure the device is connected and recognized by the system.")

        controls_output = subprocess.check_output(["amixer", f"-D{device_string}", "controls"], text=True, stderr=subprocess.DEVNULL)
        volume_match = re.search(r"numid=(\d+),iface=MIXER,name='(.*Master Playback Volume.*)'", controls_output)
        if volume_match:
            volume_control = volume_match.group(2)
            print(f"Detected Volume Control: {volume_control}")
        else:
            raise RuntimeError("Volume control not found.")

        eq_match = re.search(r"numid=(\d+),iface=MIXER,name='EQIIR1\.0 eqiir_coef_1'", controls_output)
        if eq_match:
            eq_numid = eq_match.group(1)
            print(f"Detected EQ numid: {eq_numid}")
        else:
            raise RuntimeError("EQ control not found.")

    except subprocess.CalledProcessError as e:
        print(f"Failed to run device detection commands: {e}")
        raise
    except RuntimeError as e:
        print(e)
        raise

def scale_volume(user_volume):
    normalized_volume = user_volume / 100.0
    scaled_volume = 31 * math.sqrt(normalized_volume)
    return int(round(scaled_volume))

def scan_for_files(directory_name: str, file_extension: str, extra_paths: list = None):
    found_files = []
    dir_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), directory_name)

    try:
        if os.path.exists(dir_path):
            found_files.extend([f for f in os.listdir(dir_path) if f.endswith(file_extension)])
        else:
            print(f"Error: The '{directory_name}' directory is missing. It should be located in the same folder as this script.")

        if extra_paths:
            for path in extra_paths:
                if os.path.exists(path):
                    found_files.extend([f for f in os.listdir(path) if f.endswith(file_extension)])
                else:
                    print(f"Warning: The directory '{path}' does not exist.")
    except Exception as e:
        print(f"Error scanning for files: {e}")

    return found_files

def execute_command(command: str, data: int = 0, file_name: str = None):
    if device_string is None or volume_control is None or eq_numid is None:
        raise RuntimeError("Device not initialized. Call initialize_device() first.")

    command_switch = {
        'volume': lambda x: handle_volume(data),
        'eq': lambda x: handle_eq(file_name),
        'play': lambda x: handle_play(file_name),
        'pause': lambda x: handle_pause(),
        'stop': lambda x: handle_stop(),
        'mute': lambda x: handle_mute(),
        'record': lambda x: handle_record(start=data, filename=file_name)
    }

    command_function = command_switch.get(command, lambda x: handle_unknown_command(data))
    try:
        command_function(data)
    except Exception as e:
        print(f"Error executing command '{command}': {e}")

def handle_volume(data: int):
    amixer_command = f"amixer -D{device_string} cset name='{volume_control}' {data}"
    try:
        subprocess.run(amixer_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError as e:
        print(f"Failed to set volume: {e}")

def handle_eq(eq_file_name: str):
    if not eq_file_name:
        print("No EQ file name provided.")
        return

    ctl_command = f"./sof-ctl -D{device_string} -n {eq_numid} -s {eq_file_name}"
    try:
        subprocess.run(ctl_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError as e:
        print(f"Failed to apply EQ settings: {e}")

def handle_play(play_file_name: str):
    global aplay_process, paused, current_file

    if not play_file_name:
        print("No file name provided for playback.")
        return

    try:
        if paused and current_file == play_file_name:
            os.kill(aplay_process.pid, signal.SIGCONT)
            print("Playback resumed.")
            paused = False
            return

        if aplay_process is not None:
            if aplay_process.poll() is None:
                if current_file == play_file_name:
                    print("Playback is already in progress.")
                    return
                else:
                    os.kill(aplay_process.pid, signal.SIGKILL)
                    print("Stopping current playback to play a new file.")
            else:
                print("Previous process is not running, starting new playback.")

        default_audio_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'audios')
        file_path = next((os.path.join(path, play_file_name) for path in [default_audio_dir] + extra_audio_paths if os.path.exists(os.path.join(path, play_file_name))), None)

        if file_path is None:
            print(f"Error: File '{play_file_name}' not found in the default 'audios' directory or any provided paths.")
            return

        aplay_command = f"aplay -D{device_string} '{file_path}'"
        aplay_process = subprocess.Popen(aplay_command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        current_file = play_file_name
        print(f"Playing file: {play_file_name}.")
        paused = False
    except subprocess.CalledProcessError as e:
        print(f"Failed to play file: {e}")
    except Exception as e:
        print(f"Error during playback: {e}")

def handle_pause():
    global aplay_process, paused

    if aplay_process is None:
        print("No playback process to pause.")
        return

    if aplay_process.poll() is not None:
        print("Playback process has already finished.")
        return

    try:
        os.kill(aplay_process.pid, signal.SIGSTOP)
        paused = True
        print("Playback paused.")
    except Exception as e:
        print(f"Failed to pause playback: {e}")

def handle_stop():
    global aplay_process, paused, current_file

    if aplay_process is None:
        print("No playback process to stop.")
        return

    if aplay_process.poll() is not None:
        print("Playback process has already finished.")
        return

    try:
        os.kill(aplay_process.pid, signal.SIGKILL)
        aplay_process = None
        paused = False
        current_file = None
        print("Playback stopped.")
    except Exception as e:
        print(f"Failed to stop playback: {e}")

def handle_mute():
    if not is_muted:
        previous_volume = scale_volume(50)  # Assume 50 is the current volume
        handle_volume(0)
        is_muted = True
        print("Audio muted.")
    else:
        handle_volume(previous_volume)
        is_muted = False
        print("Audio unmuted.")

def handle_record(start: bool, filename: str):
    global arecord_process

    if start:
        if arecord_process is not None and arecord_process.poll() is None:
            print("Recording is already in progress.")
            return

        if not filename:
            print("No filename provided for recording.")
            return

        record_command = f"arecord -D{device_string} -f cd -t wav {filename}"
        try:
            arecord_process = subprocess.Popen(record_command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            print(f"Started recording: {filename}")
        except subprocess.CalledProcessError as e:
            print(f"Failed to start recording: {e}")

    else:
        if arecord_process is None or arecord_process.poll() is not None:
            print("No recording process to stop.")
            return

        try:
            os.kill(arecord_process.pid, signal.SIGINT)
            arecord_process = None
            print("Stopped recording.")
        except Exception as e:
            print(f"Failed to stop recording: {e}")

def handle_unknown_command(data: int):
    print(f"Unknown command: {data}")



# SPDX-License-Identifier: BSD-3-Clause

import os
import sof_controller_engine as sof_ctl
import argparse

os.chdir(os.path.dirname(os.path.abspath(__file__)))

class TextBasedUI:
    def __init__(self, audio_paths, config_paths):
        self.selected_wav_file = None
        self.selected_eq_file = None
        self.volume = 15
        self.is_playing = False
        self.is_recording = False
        self.scan_and_populate_files(audio_paths, config_paths)
        sof_ctl.initialize_device()
        self.run_ui()

    def scan_and_populate_files(self, audio_paths, config_paths):
        self.wav_files = sof_ctl.scan_for_files('audios', '.wav', extra_paths=audio_paths)

        if self.wav_files:
            self.selected_wav_file = self.wav_files[0]
            print(f"Default WAV file selected: {self.selected_wav_file}")

        self.eq_files = sof_ctl.scan_for_files('eq_configs', '.txt', extra_paths=config_paths)

        if self.eq_files:
            self.selected_eq_file = self.eq_files[0]
            print(f"Default EQ config selected: {self.selected_eq_file}")

    def display_menu(self):
        print("\nSOF Text-Based UI")
        print("==================")
        print("1. Select a WAV file")
        if self.selected_wav_file:
            print(f"   Currently selected: {self.selected_wav_file}")
        else:
            print("   No WAV file selected")

        print("2. Select an EQ Config")
        if self.selected_eq_file:
            print(f"   Currently selected: {self.selected_eq_file}")
        else:
            print("   No EQ file selected")

        print(f"3. Set Volume (Current: {self.volume})")
        print(f"4. Play/Pause (Current: {'Playing' if self.is_playing else 'Paused'})")
        print(f"5. Start/Stop Recording (Current: {'Recording' if self.is_recording else 'Not Recording'})")
        print("6. Apply EQ Config")
        print("7. Quit")
        print("==================")

    def run_ui(self):
        while True:
            self.display_menu()
            choice = input("Enter your choice (1-7): ").strip()

            if choice == '1':
                self.select_wav_file()
            elif choice == '2':
                self.select_eq_file()
            elif choice == '3':
                self.set_volume()
            elif choice == '4':
                self.toggle_play_pause()
            elif choice == '5':
                self.toggle_record()
            elif choice == '6':
                self.apply_eq_config()
            elif choice == '7':
                print("Exiting...")
                break
            else:
                print("Invalid choice, please try again.")

    def select_wav_file(self):
        if not self.wav_files:
            print("No .wav files found in the current directory.")
            return

        print("\nSelect a WAV file:")
        for idx, wav_file in enumerate(self.wav_files):
            print(f"{idx + 1}. {wav_file}")

        file_choice = input(f"Enter the number of the file (1-{len(self.wav_files)}): ").strip()

        try:
            file_index = int(file_choice) - 1
            if 0 <= file_index < len(self.wav_files):
                self.selected_wav_file = self.wav_files[file_index]
                print(f"Selected file: {self.selected_wav_file}")
            else:
                print("Invalid selection.")
        except ValueError:
            print("Invalid input, please enter a number.")

    def select_eq_file(self):
        if not self.eq_files:
            print("No EQ config files found in the eq_configs directory.")
            return

        print("\nSelect an EQ Config file:")
        for idx, eq_file in enumerate(self.eq_files):
            print(f"{idx + 1}. {eq_file}")

        file_choice = input(f"Enter the number of the file (1-{len(self.eq_files)}): ").strip()

        try:
            file_index = int(file_choice) - 1
            if 0 <= file_index < len(self.eq_files):
                self.selected_eq_file = self.eq_files[file_index]
                print(f"Selected EQ Config: {self.selected_eq_file}")
            else:
                print("Invalid selection.")
        except ValueError:
            print("Invalid input, please enter a number.")

    def set_volume(self):
        volume_input = input("Enter volume level (0-100): ").strip()

        try:
            user_volume = int(volume_input)
            if 0 <= user_volume <= 100:
                self.volume = user_volume
                scaled_volume = sof_ctl.scale_volume(user_volume)
                sof_ctl.execute_command(command="volume", data=scaled_volume)
                print(f"Volume set to {self.volume}.")
            else:
                print("Volume must be between 0 and 100.")
        except ValueError:
            print("Invalid input, please enter a number.")

    def toggle_play_pause(self):
        if not self.selected_wav_file:
            print("No WAV file selected.")
            return

        if not self.is_playing:
            sof_ctl.execute_command(command="play", file_name=self.selected_wav_file)
            self.is_playing = True
            print("Playing...")
        else:
            sof_ctl.execute_command(command="pause")
            self.is_playing = False
            print("Paused.")

    def toggle_record(self):
        recordings_dir = os.path.join(os.getcwd(), 'recordings')

        if not self.is_recording:
            filename = input(f"Enter filename to save the recording (default: {recordings_dir}/record.wav): ").strip()
            if not filename:
                filename = "record.wav"

            full_path = os.path.join(recordings_dir, filename)
            sof_ctl.execute_command(command="record", data=True, file_name=full_path)
            self.is_recording = True
        else:
            sof_ctl.execute_command(command="record", data=False)
            self.is_recording = False

    def apply_eq_config(self):
        if not self.selected_eq_file:
            print("No EQ config file selected.")
            return

        eq_file_path = os.path.join(os.getcwd(), 'eq_configs', self.selected_eq_file)
        sof_ctl.execute_command(command="eq", file_name=eq_file_path)
        print(f"Applied EQ config: {self.selected_eq_file}")

def main():
    parser = argparse.ArgumentParser(description="SOF TUI Application")
    parser.add_argument('--audio-path', action='append', help="Additional path to search for audio files (.wav)")
    parser.add_argument('--config-path', action='append', help="Additional path to search for EQ config files (.txt)")
    args = parser.parse_args()

    sof_ctl.extra_audio_paths = args.audio_path if args.audio_path else []

    TextBasedUI(audio_paths=args.audio_path, config_paths=args.config_path)

if __name__ == "__main__":
    main()



# SPDX-License-Identifier: BSD-3-Clause

import gi
import os
import argparse
import sof_controller_engine as sof_ctl
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk

class MyWindow(Gtk.Window):
    def __init__(self, audio_paths, config_paths):
        super().__init__(title="SOF Demo Gui")
        self.set_resizable(False)
        self.set_border_width(10)
        self.apply_css()
        self.init_ui(audio_paths, config_paths)
        sof_ctl.initialize_device()
        self.is_muted = False
        self.previous_volume = 50  # Store the previous volume level

    def apply_css(self):
        css_provider = Gtk.CssProvider()
        css = """
        * {
            font-family: "Segoe UI", "Arial", sans-serif;
            font-size: 14px;
            color: #020202;
        }
        window {
            background-color: #2C3E50;
        }
        frame {
            border: 1px solid #34495E;
            border-radius: 5px;
            padding: 10px;
            margin: 5px;
            background-color: #34495E;
        }
        button, togglebutton {
            background-color: #3b3b3b;
            border: none;
            padding: 10px;
            color: #020202;
        }
        button:hover, togglebutton:hover {
            background-color: #A9A9A9;
            color: #020202;
        }
        togglebutton:checked {
            background-color: #A9A9A9;
            color: #020202;
        }
        scale {
            background-color: #34495E;
            border-radius: 5px;
            color: #FFFFFF;
        }
        label {
            color: #020202;
        }
        headerbar {
            background-color: #2C3E50;
            color: #FFFFFF;
        }
        headerbar.titlebar {
            background-color: #2C3E50;
        }
        """
        css_provider.load_from_data(css.encode('utf-8'))
        screen = Gdk.Screen.get_default()
        Gtk.StyleContext.add_provider_for_screen(screen, css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

    def init_ui(self, audio_paths, config_paths):
        main_grid = Gtk.Grid()
        main_grid.set_row_spacing(10)
        main_grid.set_column_spacing(10)
        self.add(main_grid)

        control_frame = self.create_control_frame()
        main_grid.attach(control_frame, 0, 0, 1, 1)

        file_frame = self.create_file_frame()
        main_grid.attach(file_frame, 0, 1, 1, 1)

        record_frame = self.create_record_frame()
        main_grid.attach(record_frame, 0, 2, 1, 1)

        self.scan_and_populate_dropdowns(audio_paths, config_paths)

    def create_control_frame(self):
        control_frame = Gtk.Frame(label="Playback and Volume Control")
        control_grid = Gtk.Grid()
        control_grid.set_row_spacing(10)
        control_grid.set_column_spacing(10)
        control_frame.add(control_grid)

        volume_adjustment = Gtk.Adjustment(value=100, lower=0, upper=100, step_increment=1, page_increment=5, page_size=0)
        self.volume_button = Gtk.Scale(name="volume", orientation=Gtk.Orientation.HORIZONTAL, adjustment=volume_adjustment)
        self.volume_button.set_digits(0)
        self.volume_button.set_hexpand(True)
        self.volume_button.connect("value-changed", self.on_volume_changed)
        control_grid.attach(self.volume_button, 0, 0, 2, 1)

        self.play_pause_button = Gtk.ToggleButton(label="Play")
        self.play_pause_button.connect("toggled", self.on_play_pause_toggled)
        control_grid.attach(self.play_pause_button, 0, 1, 1, 1)

        self.stop_button = Gtk.Button(label="Stop")
        self.stop_button.connect("clicked", self.on_stop_clicked)
        control_grid.attach(self.stop_button, 1, 1, 1, 1)

        self.mute_button = Gtk.Button(label="Mute")
        self.mute_button.connect("clicked", self.on_mute_clicked)
        control_grid.attach(self.mute_button, 0, 2, 2, 1)

        return control_frame

    def create_file_frame(self):
        file_frame = Gtk.Frame(label="File Selection")
        file_grid = Gtk.Grid()
        file_grid.set_row_spacing(10)
        file_grid.set_column_spacing(10)
        file_frame.add(file_grid)

        self.wav_dropdown = Gtk.ComboBoxText()
        self.wav_dropdown.connect("changed", self.on_wav_file_selected)
        wav_label = Gtk.Label(label="Select WAV File")
        wav_label.set_margin_top(5)
        wav_label.set_margin_bottom(5)
        file_grid.attach(wav_label, 0, 0, 1, 1)
        file_grid.attach(self.wav_dropdown, 1, 0, 1, 1)

        self.eq_dropdown = Gtk.ComboBoxText()
        eq_label = Gtk.Label(label="Select EQ Config")
        eq_label.set_margin_top(5)
        eq_label.set_margin_bottom(5)
        file_grid.attach(eq_label, 0, 1, 1, 1)
        file_grid.attach(self.eq_dropdown, 1, 1, 1, 1)

        self.apply_eq_button = Gtk.Button(label="Apply EQ Config")
        self.apply_eq_button.connect("clicked", self.on_apply_eq_clicked)
        file_grid.attach(self.apply_eq_button, 1, 2, 1, 1)

        return file_frame

    def create_record_frame(self):
        record_frame = Gtk.Frame(label="Recording Control")
        record_grid = Gtk.Grid()
        record_grid.set_row_spacing(10)
        record_grid.set_column_spacing(10)
        record_grid.set_hexpand(True)
        record_grid.set_vexpand(True)
        record_frame.add(record_grid)

        self.record_button = Gtk.ToggleButton(label="Record")
        self.record_button.connect("toggled", self.on_record_toggled)
        self.record_button.set_hexpand(True)
        self.record_button.set_vexpand(True)
        record_grid.attach(self.record_button, 0, 0, 1, 1)

        self.record_index = 1

        return record_frame

    def scan_and_populate_dropdowns(self, audio_paths, config_paths):
        wav_files = sof_ctl.scan_for_files('audios', '.wav', extra_paths=audio_paths)

        self.wav_dropdown.remove_all()
        for wav_file in wav_files:
            self.wav_dropdown.append_text(wav_file)

        if wav_files:
            self.wav_dropdown.set_active(0)
            self.selected_wav_file = wav_files[0]

        eq_files = sof_ctl.scan_for_files('eq_configs', '.txt', extra_paths=config_paths)

        self.eq_dropdown.remove_all()
        for eq_file in eq_files:
            self.eq_dropdown.append_text(eq_file)

        if eq_files:
            self.eq_dropdown.set_active(0)
            self.selected_eq_file = eq_files[0]

    def on_volume_changed(self, widget):
        user_volume = widget.get_value()
        scaled_volume = sof_ctl.scale_volume(user_volume)
        sof_ctl.execute_command(command="volume", data=scaled_volume)

    def on_play_pause_toggled(self, widget):
        if widget.get_active():
            widget.set_label("Pause")
            sof_ctl.execute_command(command="play", file_name=self.selected_wav_file)
        else:
            widget.set_label("Play")
            sof_ctl.execute_command(command="pause")

    def on_stop_clicked(self, widget):
        self.play_pause_button.set_active(False)
        sof_ctl.execute_command(command="stop")

    def on_mute_clicked(self, widget):
        if not self.is_muted:
            self.previous_volume = self.volume_button.get_value()
            self.volume_button.set_value(0)
            self.is_muted = True
            widget.set_label("Unmute")
        else:
            self.volume_button.set_value(self.previous_volume)
            self.is_muted = False
            widget.set_label("Mute")

    def on_record_toggled(self, widget):
        if widget.get_active():
            widget.set_label("Stop Recording")
            filename = self.generate_sequential_filename()
            sof_ctl.execute_command(command="record", data=True, file_name=filename)
        else:
            widget.set_label("Record")
            sof_ctl.execute_command(command="record", data=False)

    def generate_sequential_filename(self):
        recordings_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'recordings')

        if not os.path.exists(recordings_dir):
            os.makedirs(recordings_dir)

        filename = os.path.join(recordings_dir, f"recording_{self.record_index}.wav")
        self.record_index += 1

        return filename

    def on_wav_file_selected(self, widget):
        self.selected_wav_file = widget.get_active_text()

    def on_apply_eq_clicked(self, widget):
        self.selected_eq_file = self.eq_dropdown.get_active_text()
        if self.selected_eq_file:
            eq_file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'eq_configs', self.selected_eq_file)
            sof_ctl.execute_command(command="eq", file_name=eq_file_path)

def main():
    parser = argparse.ArgumentParser(description="SOF GUI Application")
    parser.add_argument('--audio-path', action='append', help="Additional path to search for audio files (.wav)")
    parser.add_argument('--config-path', action='append', help="Additional path to search for EQ config files (.txt)")
    args = parser.parse_args()

    sof_ctl.extra_audio_paths = args.audio_path if args.audio_path else []

    win = MyWindow(audio_paths=args.audio_path, config_paths=args.config_path)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    Gtk.main()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright (c) 2019, Intel Corporation. All rights reserved.
#
# Author: Janusz Jankowski <janusz.jankowski@linux.intel.com>

# Loads current config file and saves minimal configuration to
# the given file.
#
# Usage:
#   savedefconfig.py KCONFIG_FILENAME DEFCONFIG_OUTPUT_FILE

import os
import sys

import kconfiglib


def main():
    if len(sys.argv) < 3:
        sys.exit("usage: {} KCONFIG_FILENAME DEFCONFIG_OUTPUT_FILE"
            .format(sys.argv[0]))

    kconf = kconfiglib.Kconfig(sys.argv[1])
    kconf.load_config()
    kconf.write_min_config(sys.argv[2])


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# SPDX-License-Identifier: ISC
# Copyright (c) 2018, Ulf Magnusson

# Works like 'make alldefconfig'. Verified by the test suite to generate
# identical output to 'make alldefconfig' for all ARCHes.
#
# The default output filename is '.config'. A different filename can be passed
# in the KCONFIG_CONFIG environment variable.
#
# Usage for the Linux kernel:
#
#   $ make [ARCH=<arch>] scriptconfig SCRIPT=Kconfiglib/alldefconfig.py

import kconfiglib

def main():
    kconf = kconfiglib.standard_kconfig()
    kconfiglib.load_allconfig(kconf, "alldef.config")
    kconf.write_config()

if __name__ == "__main__":
    main()
